def convert_sec_to_hhmmss_format(sec):
    """
    초를 hh:mm:ss 포맷으로 변환
    """

    hh = sec // 3600
    mm = (sec % 3600) // 60
    ss = sec % 60

    if hh == 0:
        return f"{mm}:{ss:02}"
    else:
        return f"{hh}:{mm:02}:{ss:02}"


        
def convert_hhmmss_format_to_sec(hhmmss):
    """
    hh:mm:ss 포맷을 초로 변경
    """

    if ":" not in hhmmss:
        return hhmmss

    # hh:mm:ss format -> seconds
    hms = hhmmss.split(":")

    unit = 1
    seconds = 0
    for t in hms[::-1]:
        seconds += int(t)*unit
        unit *= 60

    return seconds


def convert_hm_format_to_min(hm):
    """
    00h 00m 포맷을 분으로 변경
    """

    hours = hm.split('h')[0]
    mins = hm.split('h')[1].split('m')[0]
    if mins == "":
        mins = 0

    return float(hours)*60 + float(mins)



def convert_iso_8859_to_sec(iso:str) -> int:
    """
    convert iso 8601 time format to seconds
    """

    sec = 0
    cur_type = "date"
    cur_num = ""

    for c in iso:
        if c == "P":
            pass
        elif c == "T":
            cur_type = "time"

        elif c in ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"]:
            cur_num += c
        
        elif c == "Y":
            sec += int(cur_num) * 365 * 24 * 3600
            cur_num = ""

        elif c == "M":
            if cur_type == "date":
                sec += int(cur_num) * 30 * 24 * 3600
            else:
                sec += int(cur_num) * 60

            cur_num = ""
        
        elif c == "D":
            sec += int(cur_num) * 24 * 3600
            cur_num = ""

        elif c == "H":
            sec += int(cur_num) * 3600
            cur_num = ""
        
        elif c == "S":
            sec += int(cur_num)
            cur_num = ""


    return int(sec)


import email
import imaplib
from email.header import decode_header, make_header


class IMAP:

    def __init__(self, login_id, login_pwd):
        self.__imap = imaplib.IMAP4_SSL('imap.gmail.com')

        self.__imap.login(login_id, login_pwd)
        self.__imap.select("INBOX")

    def get_recent_email(self, search_mail_address):

        if search_mail_address != "ALL":
            search_range = f'(FROM "{search_mail_address}")'
        else:
            search_range = 'ALL'

        # 사서함의 모든 메일의 uid 정보 가져오기
        status, messages = self.__imap.uid('search', None, search_range)
        messages = messages[0].split()

        # 0이 가장 마지막 메일, -1이 가장 최신 메일
        recent_email = messages[-1]

        # fetch 명령어로 메일 가져오기
        res, msg = self.__imap.uid('fetch', recent_email, "(RFC822)")

        # 사람이 읽을 수 있는 형태로 변환
        raw = msg[0][1].decode('utf-8')
        
        return Email(raw)


class Email:

    def __init__(self, raw):
        self.__raw = raw
        self.__email_message = email.message_from_string(self.__raw)
        self.__from_address = make_header(decode_header(self.__email_message.get('From')))
        self.__subject = make_header(decode_header(self.__email_message.get('Subject')))
        
        self.__set_body()

    def __set_body(self):
        if self.__email_message.is_multipart():
            for part in self.__email_message.walk():
                ctype = part.get_content_type()
                cdispo = str(part.get('Content-Disposition'))
                if ctype == 'text/plain' and 'attachment' not in cdispo:
                    self.__body = part.get_payload(decode=True)  # decode
                    break
        else:
            self.__body = self.__email_message.get_payload(decode=True)
            
        self.__body = self.__body.decode('utf-8')


    def get_from_address(self):
        return self.__from_address


    def get_subject(self):
        return self.__subject

    
    def get_body(self):
        return self.__body




from __future__ import annotations

import re
import json
import gspread
import pandas as pd

from gspread.exceptions import APIError, WorksheetNotFound

from googleapiclient.discovery import build

from oauth2client.service_account import ServiceAccountCredentials


scope = ["https://spreadsheets.google.com/feeds", "https://www.googleapis.com/auth/drive"]


class GspreadConnection:

    def __init__(self, key_path, scope=scope):
        self.__scope = scope
        self.__creds = ServiceAccountCredentials.from_json_keyfile_name(key_path, self.__scope)

    
    def get_spreadsheet(self, sh_id):
        """
        스프레드시트 아이디로 스프레드시트 불러오기
        """
        return SpreadSheet(sh_id, self.__creds)



class SpreadSheet:

    def __init__(self, sh_id, creds):
        self.__sh_id = sh_id
        self.__creds = creds

        try:
            self.__spreadsheet = gspread.authorize(creds).open_by_key(sh_id)

        except APIError as err:

            if err.args[0]["code"] == 403:
                print("gserviceaccount에 액세스 권한이 필요합니다")

            elif err.args[0]["code"] == 404:
                print("잘못된 시트 ID 입니다")

        except Exception as e:
            print(e)


    def get_sheet_id(self):

        return self.__sh_id

    def get_creds(self):

        return self.__creds

    def get_spreadsheet(self):

        return self.__spreadsheet


    def worksheet_exists(self, worksheet_name):
        
        try:
            self.get_spreadsheet().worksheet(worksheet_name)
            return True
        except WorksheetNotFound:
            return False
        else:
            return None


    def get_worksheet(self, worksheet_name, if_not_exist:str="fail"):
        """
        구글 스프레드시트 워크시트 이름으로 워크시트 불러오기
        
        Parameter
        ----------
        
        if_not_exist: {"create", "fail"}

            How to behave if the worksheet named 'worksheet_name' is not exist
            - create : create a new worksheet named 'worksheet_name' and write values to it
            - fail : raise a ValueError
            default value is "fail"
        
        """

        assert if_not_exist in ["create", "fail"], "if_not_exist에 잘못된 값 입력 / create, fail 중 하나로 입력해주세요"

        # 시트가 없는 경우
        if self.worksheet_exists(worksheet_name=worksheet_name) is None:
            return None


        if not self.worksheet_exists(worksheet_name=worksheet_name):
            
            print(f"{self.get_worksheet.__qualname__} 실패 :: <{worksheet_name}> 이름의 시트가 존재하지 않습니다")

            if if_not_exist == "fail":
                return None
            
            # 새로 워크시트 생성
            else:
                _ = self.create_worksheet(worksheet_name)
        

        return self.__spreadsheet.worksheet(worksheet_name)



    def get_all_worksheets(self):

        return self.__spreadsheet.worksheets()
    


    def create_worksheet(self, worksheet_name:str, row_count:int=1000, col_count:int=10, index:int=0, hidden:bool=False, frozen_row_count:int=None, frozen_column_count:int=None):
        """
        새 워크시트 만들기
        """

        if self.worksheet_exists(worksheet_name=worksheet_name):
            print(f"{self.create_worksheet.__qualname__} 실패 :: <{worksheet_name}> 이름의 시트가 이미 존재합니다")
            return None


        service = build("sheets", "v4", credentials=self.get_creds())

        # set grid properties
        grid_properties = {
            "rowCount":row_count,
            "columnCount":col_count
        }
        if frozen_row_count is not None:
            grid_properties["frozenRowCount"] = frozen_row_count

        if frozen_column_count is not None:
            grid_properties["frozenColumnCount"] = frozen_column_count


        request_body = {
            "requests": [{
                "addSheet": {
                    "properties": {
                        "title": worksheet_name,
                        "index": index,
                        "hidden": hidden,
                        "gridProperties": grid_properties,
                    }
                }
            }]
        }

        sheet = service.spreadsheets()
        response = sheet.batchUpdate(
            spreadsheetId=self.get_sheet_id(),
            body=request_body
        ).execute()

        return response



    def duplicate_worksheet(self, worksheet_name, dup_sheet_name):
        """
        기존 워크시트 복제하기
        """
        
        # 복제할 시트가 없는 경우
        if not self.worksheet_exists(worksheet_name=dup_sheet_name):
        
            print(f"{self.duplicate_worksheet.__qualname__} 실패 :: <{dup_sheet_name}> 이름의 시트가 존재하지 않습니다")
            return None
        
        # 결과 시트 이름이 이미존재하는 경우
        if self.worksheet_exists(worksheet_name=worksheet_name):

            print(f"{self.duplicate_worksheet.__qualname__} 실패 :: <{worksheet_name}> 이름의 시트가 이미 존재합니다")
            return None


        dup_worksheet = self.get_worksheet(worksheet_name=dup_sheet_name)

        service = build("sheets", "v4", credentials=self.get_creds())
        request_body = {
            "requests": [{
                "duplicateSheet": {
                    "sourceSheetId": dup_worksheet.id,
                    "newSheetName": worksheet_name,
                }
            }]
        }

    
        sheet = service.spreadsheets()
        response = sheet.batchUpdate(
            spreadsheetId=self.get_sheet_id(),
            body=request_body
        ).execute()

        return response



    def delete_worksheet(self, worksheet_name):

        if not self.worksheet_exists(worksheet_name=worksheet_name):
            print(f"{self.delete_worksheet.__qualname__} 실패 :: <{worksheet_name}> 이름의 시트가 존재하지 않습니다")
            return None
        

        worksheet = self.get_worksheet(worksheet_name)

        service = build("sheets", "v4", credentials=self.get_creds())
        request_body = {
            "requests": [{
                "deleteSheet": {
                    "sheetId": worksheet.id
                }
            }]
        }

        sheet = service.spreadsheets()
        response = sheet.batchUpdate(
            spreadsheetId=self.get_sheet_id(),
            body=request_body,
        ).execute()

        return response


    
    def update_worksheet_properties(self, worksheet_name, properties):
        """
        워크시트 속성 변경하기
        """

        if not self.worksheet_exists(worksheet_name=worksheet_name):
            print(f"{self.update_worksheet_properties.__qualname__} 실패 :: <{worksheet_name}> 이름의 시트가 존재하지 않습니다")
            return None


        worksheet = self.get_worksheet(worksheet_name)

        service = build("sheets", "v4", credentials=self.get_creds())
        properties.update({
            "sheetId": worksheet.id
        })

        request_body = {
            "requests": [{
                "updateSheetProperties": {
                    "properties": properties,
                    'fields': (",").join(list(properties.keys()))
                }
            }]
        }

        sheet = service.spreadsheets()
        response = sheet.batchUpdate(
            spreadsheetId=self.get_sheet_id(),
            body=request_body,
        ).execute()

        return response    



    def set_cell_format(self, worksheet_name:str, set_range:str, fontfamily:str=None, fontsize:int=None, bold:bool=None, italic:bool=None, color:dict=None, horizontal_alignment:str=None, vertical_alignment:str=None, wrap_strategy:str=None, number_format:str=None, number_pattern:str=None, background_color:dict=None):

        """
        Parameter
        ---------

        color : dict type
            {
                "red":int,
                "blue":int,
                "green":int
            }

        horizontal_alignment : {"LEFT", "RIGHT", "CENTER"}

        vertical_alignment : {"TOP", "MIDDLE", "BOTTOM"}

        wrap_strategy : {"OVERFLOW_CELL", "LEGACY_WRAP", "CLIP", "WRAP"}

        number_Format : {"TEXT", "NUMBER", "PERCENT", "CURRENCY", "DATE", "TIME", "DATE_TIME", "SCIENTIFIC"}

        background_color : dict type
            {
                "red":int,
                "blue":int,
                "green":int
            }
        """


        if not self.worksheet_exists(worksheet_name=worksheet_name):
            print(f"{self.set_cell_format.__qualname__} 실패 :: <{worksheet_name}> 이름의 시트가 존재하지 않습니다")
            return None


        worksheet = self.get_worksheet(worksheet_name=worksheet_name)

        body = {}

        ## textFormat
        text_format = {}
        if fontfamily is not None:
            text_format["fontFamily"] = fontfamily

        if fontsize is not None:
            text_format["fontSize"] = fontsize

        if bold is not None:
            text_format["bold"] = bold
        
        if italic is not None:
            text_format["italic"] = italic

        if color is not None:
            text_format["foregroundColorStyle"] = {"rgbColor":color}

        if len(text_format) > 0:
            body["textFormat"] = text_format


        if horizontal_alignment is not None:
            body["horizontalAlignment"] = horizontal_alignment

        if vertical_alignment is not None:
            body["verticalAlignment"] = vertical_alignment

        if wrap_strategy is not None:
            body["wrapStrategy"] = wrap_strategy

        if number_format is not None:
            body["numberFormat"] = {
                "type":number_format,
                "pattern":number_pattern
                }

        if background_color is not None:
            body["backgroundColorSyle"] = {"rgbColor":background_color}
        

        worksheet.format(
            set_range,
            format=body
        )



    def append_rows(self, worksheet_name:str, append_row_num:int):

        if not self.worksheet_exists(worksheet_name=worksheet_name):
            print(f"{self.update_worksheet_properties.__qualname__} 실패 :: <{worksheet_name}> 이름의 시트가 존재하지 않습니다")
            return None

        worksheet = self.get_worksheet(worksheet_name=worksheet_name)
        worksheet.add_rows(append_row_num)



    def write_values_to_sh(self, values, worksheet_name:str, start_cell:str, if_not_exist:str="fail"):

        """
        Parameter
        ----------
        
        if_not_exist: {"create", "fail"}

            How to behave if the worksheet named 'worksheet_name' is not exist
            - create : create a new worksheet named 'worksheet_name' and write values to it
            - fail : raise a ValueError
            default value is "fail"
        """

        assert if_not_exist in ["create", "fail"], "if_not_exist에 잘못된 값 입력 / create, fail 중 하나로 입력해주세요"

        if not self.worksheet_exists(worksheet_name=worksheet_name):

            if if_not_exist == "fail":
                print(f"{self.write_values_to_sh.__qualname__} 실패 :: <{worksheet_name}> 이름의 시트가 존재하지 않습니다")
                return None
            
            else:
                
                print(f"<{worksheet_name}> 이름의 시트가 존재하지 않습니다")
                self.create_worksheet(worksheet_name=worksheet_name)
                print(f"<{worksheet_name}> 시트 생성 완료")
        


        service = build("sheets", "v4", credentials=self.get_creds())
        sheet = service.spreadsheets()

        request_body = {
            "values":values
        }
        response = sheet.values().update(
            spreadsheetId=self.get_sheet_id(),
            range=f"{worksheet_name}!{start_cell}",
            valueInputOption="USER_ENTERED",
            body=request_body,
        ).execute()

        return response



    def write_df_to_sh(self, df, worksheet_name, include_header=False, start_cell=None, date_columns=[], datetime_columns=[], to_str_columns=[], if_not_exist:str="fail"):
        """
        write data to worksheet of which name is "worksheet_name"
        데이터 프레임 구글 시트에 기록하기

        Parameter
        ----------
        
        if_not_exist: {"create", "fail"}

            How to behave if the worksheet named 'worksheet_name' is not exist
            - create : create a new worksheet named 'worksheet_name' and write values to it
            - fail : raise a ValueError
            default value is "fail"
        """

        df = df.fillna("")
        
        for col in date_columns:
            df[col] = df[col].apply(lambda x: x.strftime("%Y-%m-%d"))
        
        for col in datetime_columns:
            df[col] = df[col].apply(lambda x: x.strftime("%Y-%m-%d %H:%M:%S"))

        for col in to_str_columns:
            df[col] = df[col].astype(str)
            
        try:
            if include_header:
                values = [df.columns.tolist()] + df.values.tolist()
                if start_cell is None:
                    start_cell = "A1"

            else:
                values = df.values.tolist()
                if start_cell is None:
                    start_cell = "A2"

            response = self.write_values_to_sh(values, worksheet_name, start_cell, if_not_exist=if_not_exist)
            return response

        except Exception as e:
            print(e)



    def clear_values(self, worksheet_name:str, clear_range:str):

        if not self.worksheet_exists(worksheet_name=worksheet_name):

            print(f"{self.clear_values.__qualname__} 실패 :: <{worksheet_name}> 이름의 시트가 존재하지 않습니다")
            return None
        
        service = build("sheets", "v4", credentials=self.get_creds())
        sheet = service.spreadsheets()

        response = sheet.values().clear(
            spreadsheetId=self.get_sheet_id(), 
            range=f"{worksheet_name}!{clear_range}"
            ).execute()
        
        return response
        


    def get_df_from_gspread(self, worksheet_name:str, read_range:str, header:list=None):

        if not self.worksheet_exists(worksheet_name=worksheet_name):

            print(f"{self.get_df_from_gspread.__qualname__} 실패 :: <{worksheet_name}> 이름의 시트가 존재하지 않습니다")
            return None
        

        service = build("sheets", "v4", credentials=self.get_creds())
        sheet = service.spreadsheets()

        response = sheet.values().get(
            spreadsheetId=self.get_sheet_id(),
            range=f"{worksheet_name}!{read_range}"
        ).execute()
        values = response.get("values", [])


        if not values:
            print("데이터가 존재하지 않습니다")
            return None

        if header is None:
            df = pd.DataFrame(columns=values[0], data=values[1:])
        else:
            df = pd.DataFrame(columns=header, data=values)

        return df




def convert_column_alphabet_to_num(col_alphabet):
    """
    A : 1
    """
    
    digit = 0
    col_num = 0

    for x in col_alphabet[::-1]:
        col_num += (ord(x) - 64) * (26 ** digit)
        digit += 1

    return col_num



def convert_alphabet_range_to_num_index(alphabet_range):

    alphabet_regex = re.compile("[A-Z]")
    number_reges = re.compile("[0-9]")

    start_range, end_range = alphabet_range.split(":")

    start_col = "".join(alphabet_regex.findall(start_range))
    start_col = convert_column_alphabet_to_num(start_col)-1

    start_row = int("".join(number_reges.findall(start_range))) - 1

    end_col = "".join(alphabet_regex.findall(end_range))
    end_col = None if len(end_col) == 0 else convert_column_alphabet_to_num(end_col) 
    end_row = "".join(number_reges.findall(end_range))
    end_row = None if len(end_row) == 0 else int(end_row)

    return start_col, start_row, end_col, end_row






import time
import onetimepass as otp

from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC

from sa_package.my_selenium.webdriver import MyChromeDriver

TIMEOUT = 10


def get_signed_in_google_driver(login_id, login_pwd, secret_key):
    
    driver = MyChromeDriver()

    google_login_url = "https://accounts.google.com/v3/signin/identifier?dsh=S-1659938691%3A1673181065994169&continue=https%3A%2F%2Fwww.google.co.kr%2F&ec=GAZAmgQ&hl=ko&passive=true&flowName=GlifWebSignIn&flowEntry=ServiceLogin&ifkv=AeAAQh70mrY_65UB8exoFTOAzHkwkl1Xm6EE-iuOz8j_lLTHN7iwU5HnbD5Xxz4BNtuUTDMUaggU_g"
    driver.get(google_login_url)

    # 아이디 입력
    driver.find_element(By.XPATH, '//*[@id="identifierId"]').send_keys(login_id)
    driver.find_element(By.XPATH, '//*[@id="identifierNext"]/div/button').click()
    WebDriverWait(driver, timeout=TIMEOUT).until(lambda x: x.find_element(By.XPATH, '//*[@id="password"]/div[1]/div/div[1]/input'))
    time.sleep(1)

    # 패스워드 입력
    driver.find_element(By.XPATH, '//*[@id="password"]/div[1]/div/div[1]/input').send_keys(login_pwd)
    driver.find_element(By.XPATH, '//*[@id="passwordNext"]/div/button').click()

    try:
        # 다른 인증 방법 선택
        other_auth_xpath =  '//*[@id="view_container"]/div/div/div[2]/div/div[2]/div[2]/div[2]/div/div/button/span'
        WebDriverWait(driver, timeout=TIMEOUT).until(lambda x: x.find_element(By.XPATH, other_auth_xpath))
        time.sleep(1)
        driver.find_element(By.XPATH, other_auth_xpath).click()

        # otp 인증 선택
        otp_auth_xpath =  '//*[@id="view_container"]/div/div/div[2]/div/div[1]/div/form/span/section/div/div/div[1]/ul/li[2]/div/div[2]'
        WebDriverWait(driver, timeout=TIMEOUT).until(lambda x: x.find_element(By.XPATH, otp_auth_xpath))
        time.sleep(1)
        driver.find_element(By.XPATH, otp_auth_xpath).click()

    except Exception as e:
        # 실패한 시도 횟수 많은 경우 opt 인증 선택으로 바로 이동
        otp_auth_xpath =  '//*[@id="view_container"]/div/div/div[2]/div/div[1]/div/form/span/section/div/div/div[1]/ul/li[2]/div/div[2]'
        WebDriverWait(driver, timeout=TIMEOUT).until(lambda x: x.find_element(By.XPATH, otp_auth_xpath))
        time.sleep(1)
        driver.find_element(By.XPATH, otp_auth_xpath).click()


    # 코드 입력
    code_input_xpath = '//*[@id="totpPin"]'
    WebDriverWait(driver, timeout=TIMEOUT).until(lambda x: x.find_element(By.XPATH, code_input_xpath))

    try_num = 0

    while try_num < 5:
        # otp 코드 받기
        code = otp.get_totp(secret_key)

        # otp 코드 입력
        time.sleep(1)
        driver.find_element(By.XPATH, code_input_xpath).send_keys(code)
        time.sleep(1)

        # 다음 버튼
        next_btn_xpath = '//*[@id="totpNext"]/div/button/span'
        WebDriverWait(driver, timeout=TIMEOUT).until(EC.element_to_be_clickable((By.XPATH, next_btn_xpath)))
        driver.find_element(By.XPATH, next_btn_xpath).click()
        

        while True:
            time.sleep(3)
            # 로그인 여부 확인
            account_img = driver.find_elements(By.XPATH, '//*[@id="gb"]/div/div[2]/div[2]/div/a')
            if len(account_img) > 0:
                return driver
            else:
                # 코드 잘못된 경우 확인
                err_msg = driver.find_elements(By.XPATH, '//*[@id="view_container"]/div/div/div[2]/div/div[1]/div/form/span/section/div/div/div[2]/div/div[2]/div[2]/div')
                if len(err_msg) > 0:
                    driver.find_element(By.XPATH, code_input_xpath).clear()
                    time.sleep(3)
                    try_num += 1
                    break

    driver.close()


import datetime
import pandas as pd
from googleapiclient.discovery import build
from sa_package.convert.time_format import convert_iso_8859_to_sec

YOUTUBE_API_SERVICE_NAME = "youtube"
YOUTUBE_API_VERSION = "v3"

class YoutubeApi:

    def __init__(self, api_key):
        self.__youtube = build(
            YOUTUBE_API_SERVICE_NAME,
            YOUTUBE_API_VERSION, 
            developerKey=api_key
        )


    def get_build(self):
        return self.__youtube
    

    def get_channel_detail(self, channel_id):

        search_response = self.__youtube.channels().list(
            part="id, snippet, statistics",
            fields="items(id,snippet(title),statistics(subscriberCount))",
            id=channel_id,
        ).execute()


        channel_name = None
        subscribers = None

        for item in search_response.get("items", []):
            channel_name = item["snippet"]["title"]
            subscribers = item["statistics"]["subscriberCount"]

        return {'channel_name':channel_name, 'subscribers':subscribers}



    def get_channel_videos(self, channel_id, published_after=None, published_before=None):
        
        video_df = pd.DataFrame()

        if published_before is None:
            published_before = datetime.datetime.today().strftime("%Y-%m-%dT%H:%M:%SZ")

        if published_after is None:
            published_after = "1970-01-01T00:00:00Z"

        page_token = ""
        while True:
            search_response = self.__youtube.search().list(
                part="id, snippet",
                fields="items(id,snippet(title, channelId, publishTime))",
                channelId=channel_id,
                maxResults=50,
                order="date",
                pageToken=page_token,
                publishedBefore=published_before,
                publishedAfter=published_after
            ).execute()

            for item in search_response.get("items", []):
                if item["id"]["kind"] == "youtube#video":
                    try:
                        video_df = video_df.append({
                            "video_id": item["id"]["videoId"],
                            "title": item["snippet"]["title"],
                            "channel_id": item["snippet"]["channelId"],
                            "upload_date": datetime.datetime.strptime(item["snippet"]["publishTime"], "%Y-%m-%dT%H:%M:%SZ")
                        }, ignore_index=True)
                    except Exception as e:
                        print()
                        print(item)

            next_page_token = search_response.get("nextPageToken", None)
            if next_page_token is None:
                break
            else:
                page_token = next_page_token

        return video_df



    def get_video_detail(self, video_id:str) -> dict:
        """
        return dict
            'title', 'upload_date', 'channel_id', 'channel_name', 'length'
        """

        search_response = self.__youtube.videos().list(
            part="id, snippet, contentDetails",
            id=video_id,
        ).execute()

        title = None
        upload_date = None
        channel_id = None

        for item in search_response.get("items", []):
            if item["kind"] == "youtube#video":
                title = item["snippet"]["title"]
                upload_date = datetime.datetime.strptime(item["snippet"]["publishedAt"], "%Y-%m-%dT%H:%M:%SZ")
                channel_id = item["snippet"]["channelId"]
                channel_name = item["snippet"]["channelTitle"]
                length = convert_iso_8859_to_sec(item["contentDetails"]["duration"])

        return {'title': title, 'upload_date': upload_date, 'channel_id': channel_id, 'channel_name':channel_name, 'length':length}